@@ -2,12 +2,15 @@ pub(crate) mod drop;
 mod list;
 pub(crate) mod polars_extension;
 
-use crate::{prelude::*, PROCESS_ID};
+use std::mem;
+
 use arrow::array::{Array, FixedSizeBinaryArray};
 use arrow::bitmap::MutableBitmap;
 use arrow::buffer::Buffer;
 use polars_extension::PolarsExtension;
-use std::mem;
+
+use crate::prelude::*;
+use crate::PROCESS_ID;
 
 /// Invariants
 /// `ptr` must point to start a `T` allocation
@@ -17,7 +20,7 @@ unsafe fn create_drop<T: Sized>(mut ptr: *const u8, n_t_vals: usize) -> Box<dyn
         let t_size = std::mem::size_of::<T>() as isize;
         for _ in 0..n_t_vals {
             let _ = std::ptr::read_unaligned(ptr as *const T);
-            ptr = ptr.offset(t_size as isize)
+            ptr = ptr.offset(t_size)
         }
     })
 }
@@ -136,9 +139,10 @@ pub(crate) fn create_extension<
 
 #[cfg(test)]
 mod test {
-    use super::*;
     use std::fmt::{Display, Formatter};
 
+    use super::*;
+
     #[derive(Clone, Debug, Default, Eq, Hash, PartialEq)]
     struct Foo {
         pub a: i32,
