@@ -1,11 +1,11 @@
-use crate::prelude::*;
-
 use polars_arrow::prelude::FromData;
 #[cfg(feature = "random")]
 use rand::prelude::SliceRandom;
 #[cfg(feature = "random")]
 use rand::{rngs::SmallRng, thread_rng, SeedableRng};
 
+use crate::prelude::*;
+
 #[derive(Copy, Clone)]
 pub enum RankMethod {
     Average,
@@ -50,17 +50,27 @@ pub(crate) fn rank(s: &Series, method: RankMethod, reverse: bool) -> Series {
         _ => {}
     }
 
-    // Currently, nulls tie with the minimum or maximum bound for a type, depending on reverse.
-    // TODO: Need to expose nulls_last in argsort to prevent this.
     if s.null_count() > 0 {
-        // Fill using MaxBound/MinBound to keep nulls first.
+        let nulls = s.is_not_null().rechunk();
+        let arr = nulls.downcast_iter().next().unwrap();
+        let validity = arr.values();
+        // Currently, nulls tie with the minimum or maximum bound for a type, depending on reverse.
+        // TODO: Need to expose nulls_last in argsort to prevent this.
+        // Fill using MaxBound/MinBound to give nulls last rank.
+        // we will replace them later.
         let null_strategy = if reverse {
-            FillNullStrategy::MaxBound
-        } else {
             FillNullStrategy::MinBound
+        } else {
+            FillNullStrategy::MaxBound
         };
         let s = s.fill_null(null_strategy).unwrap();
-        return rank(&s, method, reverse);
+
+        let mut out = rank(&s, method, reverse);
+        unsafe {
+            let arr = &mut out.chunks_mut()[0];
+            *arr = arr.with_validity(Some(validity.clone()))
+        }
+        return out;
     }
 
     // See: https://github.com/scipy/scipy/blob/v1.7.1/scipy/stats/stats.py#L8631-L8737
@@ -145,7 +155,7 @@ pub(crate) fn rank(s: &Series, method: RankMethod, reverse: bool) -> Series {
             let rng = &mut SmallRng::from_rng(&mut thread_rng).unwrap();
 
             // Shuffle sort_idx positions which point to ties in the original series.
-            for i in 0..(ties_indices.len() - 1) as usize {
+            for i in 0..(ties_indices.len() - 1) {
                 let ties_index_start = ties_indices[i];
                 let ties_index_end = ties_indices[i + 1];
                 if ties_index_end - ties_index_start > 1 {
@@ -187,6 +197,8 @@ pub(crate) fn rank(s: &Series, method: RankMethod, reverse: bool) -> Series {
             //     if method == 'min':
             //         return count[dense - 1] + 1
             // ```
+            // INVALID LINT REMOVE LATER
+            #[allow(clippy::bool_to_int_with_if)]
             let mut cumsum: IdxSize = if let RankMethod::Min = method { 0 } else { 1 };
 
             dense.push(cumsum);
@@ -270,7 +282,7 @@ mod test {
     use super::*;
 
     #[test]
-    fn test_rank() -> Result<()> {
+    fn test_rank() -> PolarsResult<()> {
         let s = Series::new("a", &[1, 2, 3, 2, 2, 3, 0]);
 
         let out = rank(&s, RankMethod::Ordinal, false)
@@ -331,13 +343,13 @@ mod test {
         assert_eq!(
             out,
             &[
-                Some(4.0f32),
-                Some(5.5),
-                Some(7.0),
-                Some(5.5),
-                Some(1.5),
-                Some(1.5),
-                Some(3.0)
+                Some(2.0f32),
+                Some(3.5),
+                Some(5.0),
+                Some(3.5),
+                None,
+                None,
+                Some(1.0)
             ]
         );
         let s = Series::new(
@@ -355,15 +367,27 @@ mod test {
         );
         let out = rank(&s, RankMethod::Max, false)
             .idx()?
-            .into_no_null_iter()
+            .into_iter()
             .collect::<Vec<_>>();
-        assert_eq!(out, &[5, 6, 4, 1, 8, 4, 2, 7]);
+        assert_eq!(
+            out,
+            &[
+                Some(4),
+                Some(5),
+                Some(3),
+                None,
+                Some(7),
+                Some(3),
+                Some(1),
+                Some(6)
+            ]
+        );
 
         Ok(())
     }
 
     #[test]
-    fn test_rank_all_null() -> Result<()> {
+    fn test_rank_all_null() -> PolarsResult<()> {
         let s = UInt32Chunked::new("", &[None, None, None]).into_series();
         let out = rank(&s, RankMethod::Average, false)
             .f32()?
@@ -388,13 +412,13 @@ mod test {
     }
 
     #[test]
-    fn test_rank_reverse() -> Result<()> {
+    fn test_rank_reverse() -> PolarsResult<()> {
         let s = Series::new("", &[None, Some(1), Some(1), Some(5), None]);
         let out = rank(&s, RankMethod::Dense, true)
             .idx()?
-            .into_no_null_iter()
+            .into_iter()
             .collect::<Vec<_>>();
-        assert_eq!(out, &[1 as IdxSize, 3, 3, 2, 1]);
+        assert_eq!(out, &[None, Some(2 as IdxSize), Some(2), Some(1), None]);
 
         Ok(())
     }
