@@ -1,10 +1,11 @@
-use crate::prelude::*;
 use arrow::array::StructArray;
 
+use crate::prelude::*;
+
 impl TryFrom<StructArray> for DataFrame {
     type Error = PolarsError;
 
-    fn try_from(arr: StructArray) -> Result<Self> {
+    fn try_from(arr: StructArray) -> PolarsResult<Self> {
         let (fld, arrs, nulls) = arr.into_data();
         if nulls.is_some() {
             return Err(PolarsError::ComputeError(
@@ -19,7 +20,7 @@ impl TryFrom<StructArray> for DataFrame {
                 // reported data type is correct
                 unsafe { Series::try_from_arrow_unchecked(&fld.name, vec![arr], fld.data_type()) }
             })
-            .collect::<Result<Vec<_>>>()?;
+            .collect::<PolarsResult<Vec<_>>>()?;
         DataFrame::new(columns)
     }
 }
