@@ -16,6 +16,17 @@ fn any_values_to_utf8(avs: &[AnyValue]) -> Utf8Chunked {
         .collect_trusted()
 }
 
+#[cfg(feature = "dtype-binary")]
+fn any_values_to_binary(avs: &[AnyValue]) -> BinaryChunked {
+    avs.iter()
+        .map(|av| match av {
+            AnyValue::Binary(s) => Some(*s),
+            AnyValue::BinaryOwned(s) => Some(&**s),
+            _ => None,
+        })
+        .collect_trusted()
+}
+
 fn any_values_to_bool(avs: &[AnyValue]) -> BooleanChunked {
     avs.iter()
         .map(|av| match av {
@@ -25,6 +36,48 @@ fn any_values_to_bool(avs: &[AnyValue]) -> BooleanChunked {
         .collect_trusted()
 }
 
+fn coerce_recursively(a: &Series, dtype: &DataType) -> Series {
+    match (a.dtype(), dtype) {
+        (lhs, rhs) if lhs == rhs => a.clone(),
+        #[cfg(feature = "dtype-struct")]
+        (DataType::Struct(_), DataType::Struct(dtype_fields)) => {
+            let a = a.struct_().unwrap();
+            let mut new_fields = Vec::with_capacity(a.fields().len());
+            for (s_field, fld) in a.fields().iter().zip(dtype_fields) {
+                let mut new_s = coerce_recursively(s_field, fld.data_type());
+                if new_s.name() != fld.name {
+                    new_s.rename(&fld.name);
+                }
+                new_fields.push(new_s);
+            }
+            StructChunked::new(a.name(), &new_fields)
+                .unwrap()
+                .into_series()
+        }
+        (DataType::List(_), DataType::List(inner_type)) => {
+            let a = a.list().unwrap();
+            let a = a.rechunk();
+            let arr = a.downcast_iter().next().unwrap();
+            let s = Series::try_from(("", arr.values().clone())).unwrap();
+            let new_inner = coerce_recursively(&s, inner_type);
+            let new_values = new_inner.array_ref(0).clone();
+
+            let data_type = ListArray::<i64>::default_datatype(new_values.data_type().clone());
+            let new_arr = ListArray::<i64>::new(
+                data_type,
+                arr.offsets().clone(),
+                new_values,
+                arr.validity().cloned(),
+            );
+            Series::try_from((s.name(), Box::new(new_arr) as ArrayRef)).unwrap()
+        }
+        _ => match a.cast(dtype) {
+            Ok(s) => s,
+            _ => Series::full_null("", a.len(), dtype),
+        },
+    }
+}
+
 fn any_values_to_list(avs: &[AnyValue], inner_type: &DataType) -> ListChunked {
     // this is handled downstream. The builder will choose the first non null type
     if inner_type == &DataType::Null {
@@ -43,7 +96,7 @@ fn any_values_to_list(avs: &[AnyValue], inner_type: &DataType) -> ListChunked {
                     if b.dtype() == inner_type {
                         Some(b.clone())
                     } else {
-                        Some(Series::full_null("", b.len(), inner_type))
+                        Some(coerce_recursively(b, inner_type))
                     }
                 }
                 _ => None,
@@ -81,6 +134,8 @@ impl Series {
             DataType::Float32 => any_values_to_primitive::<Float32Type>(av).into_series(),
             DataType::Float64 => any_values_to_primitive::<Float64Type>(av).into_series(),
             DataType::Utf8 => any_values_to_utf8(av).into_series(),
+            #[cfg(feature = "dtype-binary")]
+            DataType::Binary => any_values_to_binary(av).into_series(),
             DataType::Boolean => any_values_to_bool(av).into_series(),
             #[cfg(feature = "dtype-date")]
             DataType::Date => any_values_to_primitive::<Int32Type>(av)
@@ -129,7 +184,22 @@ impl Series {
                     .unwrap()
                     .into_series());
             }
-            dtype => panic!("dtype {:?} not implemented", dtype),
+            #[cfg(feature = "object")]
+            DataType::Object(_) => {
+                use crate::chunked_array::object::registry;
+                let converter = registry::get_object_converter();
+                let mut builder = registry::get_object_builder(name, av.len());
+                for av in av {
+                    if let AnyValue::Object(val) = av {
+                        builder.append_value(val.as_any())
+                    } else {
+                        let any = converter(av.as_borrowed());
+                        builder.append_value(&*any)
+                    }
+                }
+                return Ok(builder.to_series());
+            }
+            dt => panic!("{:?} not supported", dt),
         };
         s.rename(name);
         Ok(s)
@@ -152,8 +222,9 @@ impl<'a> From<&AnyValue<'a>> for DataType {
         match val {
             Null => DataType::Null,
             Boolean(_) => DataType::Boolean,
-            Utf8(_) => DataType::Utf8,
-            Utf8Owned(_) => DataType::Utf8,
+            Utf8(_) | Utf8Owned(_) => DataType::Utf8,
+            #[cfg(feature = "dtype-binary")]
+            Binary(_) | BinaryOwned(_) => DataType::Binary,
             UInt32(_) => DataType::UInt32,
             UInt64(_) => DataType::UInt64,
             Int32(_) => DataType::Int32,
