@@ -24,7 +24,7 @@ use crate::prelude::*;
 
 macro_rules! impl_dyn_series {
     ($ca: ident, $into_logical: ident) => {
-        impl IntoSeries for $ca {
+        unsafe impl IntoSeries for $ca {
             fn into_series(self) -> Series {
                 Series(Arc::new(SeriesWrap(self)))
             }
@@ -74,8 +74,9 @@ macro_rules! impl_dyn_series {
                     .map(|ca| ca.$into_logical().into_series())
             }
 
-            fn vec_hash(&self, random_state: RandomState) -> PolarsResult<Vec<u64>> {
-                Ok(self.0.vec_hash(random_state))
+            fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
+                self.0.vec_hash(random_state, buf);
+                Ok(())
             }
 
             fn vec_hash_combine(
@@ -325,9 +326,9 @@ macro_rules! impl_dyn_series {
                 self.0.rechunk().$into_logical().into_series()
             }
 
-            fn expand_at_index(&self, index: usize, length: usize) -> Series {
+            fn new_from_index(&self, index: usize, length: usize) -> Series {
                 self.0
-                    .expand_at_index(index, length)
+                    .new_from_index(index, length)
                     .$into_logical()
                     .into_series()
             }
