@@ -74,8 +74,9 @@ macro_rules! impl_dyn_series {
                 (&self.0).into_partial_ord_inner()
             }
 
-            fn vec_hash(&self, random_state: RandomState) -> PolarsResult<Vec<u64>> {
-                Ok(self.0.vec_hash(random_state))
+            fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
+                self.0.vec_hash(random_state, buf);
+                Ok(())
             }
 
             fn vec_hash_combine(
@@ -292,8 +293,8 @@ macro_rules! impl_dyn_series {
                 self.0.rechunk().into_series()
             }
 
-            fn expand_at_index(&self, index: usize, length: usize) -> Series {
-                ChunkExpandAtIndex::expand_at_index(&self.0, index, length).into_series()
+            fn new_from_index(&self, index: usize, length: usize) -> Series {
+                ChunkExpandAtIndex::new_from_index(&self.0, index, length).into_series()
             }
 
             fn cast(&self, data_type: &DataType) -> PolarsResult<Series> {
