@@ -1,3 +1,5 @@
+#[cfg(feature = "dtype-binary")]
+mod binary;
 mod boolean;
 #[cfg(feature = "dtype-categorical")]
 mod categorical;
@@ -59,7 +61,7 @@ impl<T: PolarsDataType> Deref for SeriesWrap<ChunkedArray<T>> {
     }
 }
 
-impl<T: PolarsDataType + 'static> IntoSeries for ChunkedArray<T>
+unsafe impl<T: PolarsDataType + 'static> IntoSeries for ChunkedArray<T>
 where
     SeriesWrap<ChunkedArray<T>>: SeriesTrait,
 {
@@ -129,8 +131,9 @@ macro_rules! impl_dyn_series {
                 (&self.0).into_partial_ord_inner()
             }
 
-            fn vec_hash(&self, random_state: RandomState) -> PolarsResult<Vec<u64>> {
-                Ok(self.0.vec_hash(random_state))
+            fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
+                self.0.vec_hash(random_state, buf);
+                Ok(())
             }
 
             fn vec_hash_combine(
@@ -170,18 +173,6 @@ macro_rules! impl_dyn_series {
                 self.0.agg_list(groups)
             }
 
-            unsafe fn agg_quantile(
-                &self,
-                groups: &GroupsProxy,
-                quantile: f64,
-                interpol: QuantileInterpolOptions,
-            ) -> Series {
-                self.0.agg_quantile(groups, quantile, interpol)
-            }
-
-            unsafe fn agg_median(&self, groups: &GroupsProxy) -> Series {
-                self.0.agg_median(groups)
-            }
             fn zip_outer_join_column(
                 &self,
                 right_column: &Series,
@@ -390,8 +381,8 @@ macro_rules! impl_dyn_series {
                 self.0.rechunk().into_series()
             }
 
-            fn expand_at_index(&self, index: usize, length: usize) -> Series {
-                ChunkExpandAtIndex::expand_at_index(&self.0, index, length).into_series()
+            fn new_from_index(&self, index: usize, length: usize) -> Series {
+                ChunkExpandAtIndex::new_from_index(&self.0, index, length).into_series()
             }
 
             fn cast(&self, data_type: &DataType) -> PolarsResult<Series> {
@@ -579,6 +570,8 @@ impl<T: PolarsNumericType> private::PrivateSeriesNumeric for SeriesWrap<ChunkedA
 }
 
 impl private::PrivateSeriesNumeric for SeriesWrap<Utf8Chunked> {}
+#[cfg(feature = "dtype-binary")]
+impl private::PrivateSeriesNumeric for SeriesWrap<BinaryChunked> {}
 impl private::PrivateSeriesNumeric for SeriesWrap<ListChunked> {}
 impl private::PrivateSeriesNumeric for SeriesWrap<BooleanChunked> {
     fn bit_repr_is_large(&self) -> bool {
