@@ -145,7 +145,9 @@ impl Eq for Wrap<Series> {}
 impl Hash for Wrap<Series> {
     fn hash<H: Hasher>(&self, state: &mut H) {
         let rs = RandomState::with_seeds(0, 0, 0, 0);
-        let h = UInt64Chunked::from_vec("", self.0.vec_hash(rs).unwrap()).sum();
+        let mut h = vec![];
+        self.0.vec_hash(rs, &mut h).unwrap();
+        let h = UInt64Chunked::from_vec("", h).sum();
         h.hash(state)
     }
 }
@@ -310,7 +312,13 @@ impl Series {
         match self.dtype() {
             DataType::Float32 => Ok(self.f32().unwrap().is_nan()),
             DataType::Float64 => Ok(self.f64().unwrap().is_nan()),
-            _ => Ok(BooleanChunked::full(self.name(), false, self.len())),
+            _ => Err(PolarsError::InvalidOperation(
+                format!(
+                    "'is_nan' not supported for series with dtype {:?}",
+                    self.dtype()
+                )
+                .into(),
+            )),
         }
     }
 
@@ -319,7 +327,13 @@ impl Series {
         match self.dtype() {
             DataType::Float32 => Ok(self.f32().unwrap().is_not_nan()),
             DataType::Float64 => Ok(self.f64().unwrap().is_not_nan()),
-            _ => Ok(BooleanChunked::full(self.name(), true, self.len())),
+            _ => Err(PolarsError::InvalidOperation(
+                format!(
+                    "'is_not_nan' not supported for series with dtype {:?}",
+                    self.dtype()
+                )
+                .into(),
+            )),
         }
     }
 
@@ -330,7 +344,7 @@ impl Series {
             DataType::Float64 => Ok(self.f64().unwrap().is_finite()),
             _ => Err(PolarsError::InvalidOperation(
                 format!(
-                    "is_nan not supported for series with dtype {:?}",
+                    "'is_finite' not supported for series with dtype {:?}",
                     self.dtype()
                 )
                 .into(),
@@ -345,7 +359,7 @@ impl Series {
             DataType::Float64 => Ok(self.f64().unwrap().is_infinite()),
             _ => Err(PolarsError::InvalidOperation(
                 format!(
-                    "is_nan not supported for series with dtype {:?}",
+                    "'is_infinite' not supported for series with dtype {:?}",
                     self.dtype()
                 )
                 .into(),
