@@ -54,7 +54,7 @@ use crate::WriterFactory;
 /// use polars_io::ipc::IpcReader;
 /// use polars_io::SerReader;
 ///
-/// fn example() -> Result<DataFrame> {
+/// fn example() -> PolarsResult<DataFrame> {
 ///     let file = File::open("file.ipc").expect("file not found");
 ///
 ///     IpcReader::new(file)
@@ -76,13 +76,13 @@ pub struct IpcReader<R: MmapBytesReader> {
 
 impl<R: MmapBytesReader> IpcReader<R> {
     /// Get schema of the Ipc File
-    pub fn schema(&mut self) -> Result<Schema> {
+    pub fn schema(&mut self) -> PolarsResult<Schema> {
         let metadata = read::read_file_metadata(&mut self.reader)?;
-        Ok((&metadata.schema.fields).into())
+        Ok(metadata.schema.fields.iter().into())
     }
 
     /// Get arrow schema of the Ipc File, this is faster than creating a polars schema.
-    pub fn arrow_schema(&mut self) -> Result<ArrowSchema> {
+    pub fn arrow_schema(&mut self) -> PolarsResult<ArrowSchema> {
         let metadata = read::read_file_metadata(&mut self.reader)?;
         Ok(metadata.schema)
     }
@@ -123,14 +123,13 @@ impl<R: MmapBytesReader> IpcReader<R> {
     pub fn finish_with_scan_ops(
         mut self,
         predicate: Option<Arc<dyn PhysicalIoExpr>>,
-        aggregate: Option<&[ScanAggregation]>,
         verbose: bool,
-    ) -> Result<DataFrame> {
+    ) -> PolarsResult<DataFrame> {
         if self.memmap && self.reader.to_file().is_some() {
             if verbose {
                 eprintln!("memory map ipc file")
             }
-            match self.finish_memmapped(predicate.clone(), aggregate) {
+            match self.finish_memmapped(predicate.clone()) {
                 Ok(df) => return Ok(df),
                 Err(err) => match err {
                     PolarsError::ArrowError(e) => match e.as_ref() {
@@ -156,15 +155,7 @@ impl<R: MmapBytesReader> IpcReader<R> {
 
         let reader = read::FileReader::new(self.reader, metadata, self.projection, self.n_rows);
 
-        finish_reader(
-            reader,
-            rechunk,
-            None,
-            predicate,
-            aggregate,
-            &schema,
-            self.row_count,
-        )
+        finish_reader(reader, rechunk, None, predicate, &schema, self.row_count)
     }
 }
 
@@ -195,9 +186,9 @@ impl<R: MmapBytesReader> SerReader<R> for IpcReader<R> {
         self
     }
 
-    fn finish(mut self) -> Result<DataFrame> {
+    fn finish(mut self) -> PolarsResult<DataFrame> {
         if self.memmap && self.reader.to_file().is_some() {
-            match self.finish_memmapped(None, None) {
+            match self.finish_memmapped(None) {
                 Ok(df) => return Ok(df),
                 Err(err) => match err {
                     PolarsError::ArrowError(e) => match e.as_ref() {
@@ -229,15 +220,7 @@ impl<R: MmapBytesReader> SerReader<R> for IpcReader<R> {
 
         let ipc_reader =
             read::FileReader::new(self.reader, metadata.clone(), self.projection, self.n_rows);
-        finish_reader(
-            ipc_reader,
-            rechunk,
-            None,
-            None,
-            None,
-            &schema,
-            self.row_count,
-        )
+        finish_reader(ipc_reader, rechunk, None, None, &schema, self.row_count)
     }
 }
 
@@ -251,7 +234,7 @@ impl<R: MmapBytesReader> SerReader<R> for IpcReader<R> {
 /// use std::fs::File;
 /// use polars_io::SerWriter;
 ///
-/// fn example(df: &mut DataFrame) -> Result<()> {
+/// fn example(df: &mut DataFrame) -> PolarsResult<()> {
 ///     let mut file = File::create("file.ipc").expect("could not create file");
 ///
 ///     IpcWriter::new(&mut file)
@@ -268,7 +251,6 @@ pub struct IpcWriter<W> {
 use polars_core::frame::ArrowChunk;
 pub use write::Compression as IpcCompression;
 
-use crate::aggregations::ScanAggregation;
 use crate::mmap::MmapBytesReader;
 use crate::RowCount;
 
@@ -291,7 +273,7 @@ where
         }
     }
 
-    fn finish(&mut self, df: &mut DataFrame) -> Result<()> {
+    fn finish(&mut self, df: &mut DataFrame) -> PolarsResult<()> {
         let mut ipc_writer = write::FileWriter::try_new(
             &mut self.writer,
             &df.schema().to_arrow(),
