@@ -39,7 +39,10 @@ use polars_plan::logical_plan::optimize;
 use polars_plan::utils::{combine_predicates_expr, expr_to_leaf_column_names};
 
 use crate::physical_plan::executors::Executor;
+use crate::physical_plan::planner::create_physical_plan;
 use crate::physical_plan::state::ExecutionState;
+#[cfg(any(feature = "csv-file", feature = "parquet"))]
+use crate::physical_plan::streaming::insert_streaming_nodes;
 use crate::prelude::*;
 
 pub trait IntoLazy {
@@ -117,9 +120,9 @@ impl LazyFrame {
             slice_pushdown: false,
             // will be toggled by a scan operation such as csv scan or parquet scan
             file_caching: false,
-            aggregate_pushdown: false,
             #[cfg(feature = "cse")]
             common_subplan_elimination: false,
+            streaming: false,
         })
     }
 
@@ -155,18 +158,18 @@ impl LazyFrame {
         self
     }
 
-    /// Toggle aggregate pushdown.
-    pub fn with_aggregate_pushdown(mut self, toggle: bool) -> Self {
-        self.opt_state.aggregate_pushdown = toggle;
-        self
-    }
-
     /// Toggle slice pushdown optimization
     pub fn with_slice_pushdown(mut self, toggle: bool) -> Self {
         self.opt_state.slice_pushdown = toggle;
         self
     }
 
+    /// Allow (partial) streaming engine
+    pub fn with_streaming(mut self, toggle: bool) -> Self {
+        self.opt_state.streaming = toggle;
+        self
+    }
+
     /// Describe the logical plan.
     pub fn describe_plan(&self) -> String {
         self.logical_plan.describe()
@@ -495,14 +498,44 @@ impl LazyFrame {
         lp_arena: &mut Arena<ALogicalPlan>,
         expr_arena: &mut Arena<AExpr>,
     ) -> PolarsResult<Node> {
-        optimize(self.logical_plan, self.opt_state, lp_arena, expr_arena)
+        optimize(
+            self.logical_plan,
+            self.opt_state,
+            lp_arena,
+            expr_arena,
+            &mut vec![],
+        )
     }
 
-    fn prepare_collect(self) -> PolarsResult<(ExecutionState, Box<dyn Executor>)> {
+    fn optimize_with_scratch(
+        self,
+        lp_arena: &mut Arena<ALogicalPlan>,
+        expr_arena: &mut Arena<AExpr>,
+        scratch: &mut Vec<Node>,
+    ) -> PolarsResult<Node> {
+        optimize(
+            self.logical_plan,
+            self.opt_state,
+            lp_arena,
+            expr_arena,
+            scratch,
+        )
+    }
+
+    #[allow(unused_mut)]
+    fn prepare_collect(mut self) -> PolarsResult<(ExecutionState, Box<dyn Executor>)> {
         let file_caching = self.opt_state.file_caching;
+        let streaming = self.opt_state.streaming;
+
+        #[cfg(feature = "cse")]
+        if streaming && self.opt_state.common_subplan_elimination {
+            eprintln!("Cannot combine 'streaming' with 'common_subplan_elimination'. CSE will be turned off.");
+            self.opt_state.common_subplan_elimination = false;
+        }
         let mut expr_arena = Arena::with_capacity(256);
         let mut lp_arena = Arena::with_capacity(128);
-        let lp_top = self.optimize(&mut lp_arena, &mut expr_arena)?;
+        let mut scratch = vec![];
+        let lp_top = self.optimize_with_scratch(&mut lp_arena, &mut expr_arena, &mut scratch)?;
 
         let finger_prints = if file_caching {
             #[cfg(any(feature = "ipc", feature = "parquet", feature = "csv-file"))]
@@ -519,8 +552,18 @@ impl LazyFrame {
             None
         };
 
-        let planner = PhysicalPlanner::default();
-        let physical_plan = planner.create_physical_plan(lp_top, &mut lp_arena, &mut expr_arena)?;
+        if streaming {
+            #[cfg(any(feature = "csv-file", feature = "parquet"))]
+            {
+                insert_streaming_nodes(lp_top, &mut lp_arena, &mut expr_arena, &mut scratch)?;
+            }
+            #[cfg(not(any(feature = "csv-file", feature = "parquet")))]
+            {
+                panic!("activate feature 'csv-file' or 'parquet'")
+            }
+        }
+
+        let physical_plan = create_physical_plan(lp_top, &mut lp_arena, &mut expr_arena)?;
 
         let state = ExecutionState::with_finger_prints(finger_prints);
         Ok((state, physical_plan))
@@ -1159,38 +1202,10 @@ impl LazyFrame {
     /// inserted as columns.
     #[cfg(feature = "dtype-struct")]
     #[cfg_attr(docsrs, doc(cfg(feature = "dtype-struct")))]
-    pub fn unnest<I: IntoVec<String>>(self, cols: I) -> Self {
-        let cols = cols.into_vec();
-        self.unnest_impl(cols.into_iter().collect())
-    }
-
-    #[cfg(feature = "dtype-struct")]
-    fn unnest_impl(self, cols: PlHashSet<String>) -> Self {
-        let cols2 = cols.clone();
-        let udf_schema = move |schema: &Schema| {
-            let mut new_schema = Schema::with_capacity(schema.len() * 2);
-            for (name, dtype) in schema.iter() {
-                if cols.contains(name) {
-                    if let DataType::Struct(flds) = dtype {
-                        for fld in flds {
-                            new_schema.with_column(fld.name().clone(), fld.data_type().clone())
-                        }
-                    } else {
-                        // todo: return lazy error here.
-                        panic!("expected struct dtype")
-                    }
-                } else {
-                    new_schema.with_column(name.clone(), dtype.clone())
-                }
-            }
-            Ok(Arc::new(new_schema))
-        };
-        self.map(
-            move |df| df.unnest(&cols2),
-            Some(AllowedOptimizations::default()),
-            Some(Arc::new(udf_schema)),
-            Some("unnest"),
-        )
+    pub fn unnest<I: IntoIterator<Item = S>, S: AsRef<str>>(self, cols: I) -> Self {
+        self.map_private(FunctionNode::Unnest {
+            columns: Arc::new(cols.into_iter().map(|s| Arc::from(s.as_ref())).collect()),
+        })
     }
 }
 
@@ -1234,7 +1249,7 @@ impl LazyGroupBy {
         #[cfg(feature = "dynamic_groupby")]
         let lp = LogicalPlanBuilder::from(self.logical_plan)
             .groupby(
-                Arc::new(self.keys),
+                self.keys,
                 aggs,
                 None,
                 self.maintain_order,
@@ -1245,7 +1260,7 @@ impl LazyGroupBy {
 
         #[cfg(not(feature = "dynamic_groupby"))]
         let lp = LogicalPlanBuilder::from(self.logical_plan)
-            .groupby(Arc::new(self.keys), aggs, None, self.maintain_order)
+            .groupby(self.keys, aggs, None, self.maintain_order)
             .build();
         LazyFrame::from_logical_plan(lp, self.opt_state)
     }
