@@ -1,9 +1,6 @@
-use crate::physical_plan::state::ExecutionState;
-use crate::prelude::*;
-use polars_core::prelude::*;
 use polars_core::utils::concat_df;
-use polars_core::POOL;
-use rayon::prelude::*;
+
+use super::*;
 
 pub(crate) struct UnionExec {
     pub(crate) inputs: Vec<Box<dyn Executor>>,
@@ -11,7 +8,7 @@ pub(crate) struct UnionExec {
 }
 
 impl Executor for UnionExec {
-    fn execute(&mut self, state: &mut ExecutionState) -> Result<DataFrame> {
+    fn execute(&mut self, state: &mut ExecutionState) -> PolarsResult<DataFrame> {
         #[cfg(debug_assertions)]
         {
             if state.verbose() {
@@ -20,7 +17,17 @@ impl Executor for UnionExec {
         }
         let mut inputs = std::mem::take(&mut self.inputs);
 
-        if self.options.slice && self.options.slice_offset >= 0 {
+        let sliced_path = self.options.slice && self.options.slice_offset >= 0;
+
+        if self.options.parallel || sliced_path {
+            if state.verbose() {
+                if self.options.parallel {
+                    println!("UNION: `parallel=false` union is run sequentially")
+                } else {
+                    println!("UNION: `slice is set` union is run sequentially")
+                }
+            }
+
             let mut offset = self.options.slice_offset as usize;
             let mut len = self.options.slice_len as usize;
             let dfs = inputs
@@ -31,6 +38,10 @@ impl Executor for UnionExec {
                     state.branch_idx += idx;
                     let df = input.execute(&mut state)?;
 
+                    if !sliced_path {
+                        return Ok(Some(df));
+                    }
+
                     Ok(if offset > df.height() {
                         offset -= df.height();
                         None
@@ -50,10 +61,14 @@ impl Executor for UnionExec {
                         out
                     })
                 })
-                .collect::<Result<Vec<_>>>()?;
+                .collect::<PolarsResult<Vec<_>>>()?;
 
             concat_df(dfs.iter().flatten())
         } else {
+            if state.verbose() {
+                println!("UNION: union is run in parallel")
+            }
+
             // we don't use par_iter directly because the LP may also start threads for every LP (for instance scan_csv)
             // this might then lead to a rayon SO. So we take a multitude of the threads to keep work stealing
             // within bounds
@@ -70,9 +85,9 @@ impl Executor for UnionExec {
                                 state.branch_idx += idx;
                                 input.execute(&mut state)
                             })
-                            .collect::<Result<Vec<_>>>()
+                            .collect::<PolarsResult<Vec<_>>>()
                     })
-                    .collect::<Result<Vec<_>>>()
+                    .collect::<PolarsResult<Vec<_>>>()
             });
 
             concat_df(out?.iter().flat_map(|dfs| dfs.iter()))
