@@ -1,8 +1,10 @@
 use polars_core::prelude::*;
+#[cfg(feature = "diff")]
 use polars_core::series::ops::NullBehavior;
 use polars_ops::prelude::*;
 
 use crate::dsl::function_expr::FunctionExpr;
+use crate::prelude::function_expr::ListFunction;
 use crate::prelude::*;
 
 /// Specialized expressions for [`Series`] of [`DataType::List`].
@@ -82,10 +84,10 @@ impl ListNameSpace {
     }
 
     /// Sort every sublist.
-    pub fn sort(self, reverse: bool) -> Expr {
+    pub fn sort(self, options: SortOptions) -> Expr {
         self.0
             .map(
-                move |s| Ok(s.list()?.lst_sort(reverse).into_series()),
+                move |s| Ok(s.list()?.lst_sort(options).into_series()),
                 GetOutput::same_type(),
             )
             .with_fmt("arr.sort")
@@ -112,24 +114,19 @@ impl ListNameSpace {
     }
 
     /// Get items in every sublist by index.
-    pub fn get(self, index: i64) -> Expr {
-        self.0.map(
-            move |s| s.list()?.lst_get(index),
-            GetOutput::map_field(|field| match field.data_type() {
-                DataType::List(inner) => Field::new(field.name(), *inner.clone()),
-                dt => panic!("should be a list type, got {:?}", dt),
-            }),
-        )
+    pub fn get(self, index: Expr) -> Expr {
+        self.0
+            .map_many_private(FunctionExpr::ListExpr(ListFunction::Get), &[index], false)
     }
 
     /// Get first item of every sublist.
     pub fn first(self) -> Expr {
-        self.get(0)
+        self.get(lit(0i64))
     }
 
     /// Get last item of every sublist.
     pub fn last(self) -> Expr {
-        self.get(-1)
+        self.get(lit(-1i64))
     }
 
     /// Join all string items in a sublist and place a separator between them.
@@ -188,23 +185,22 @@ impl ListNameSpace {
     }
 
     /// Slice every sublist.
-    pub fn slice(self, offset: i64, length: usize) -> Expr {
-        self.0
-            .map(
-                move |s| Ok(s.list()?.lst_slice(offset, length).into_series()),
-                GetOutput::same_type(),
-            )
-            .with_fmt("arr.slice")
+    pub fn slice(self, offset: Expr, length: Expr) -> Expr {
+        self.0.map_many_private(
+            FunctionExpr::ListExpr(ListFunction::Slice),
+            &[offset, length],
+            false,
+        )
     }
 
     /// Get the head of every sublist
-    pub fn head(self, n: usize) -> Expr {
-        self.slice(0, n)
+    pub fn head(self, n: Expr) -> Expr {
+        self.slice(lit(0), n)
     }
 
     /// Get the tail of every sublist
-    pub fn tail(self, n: usize) -> Expr {
-        self.slice(-(n as i64), n)
+    pub fn tail(self, n: Expr) -> Expr {
+        self.slice(lit(0) - n.clone().cast(DataType::Int64), n)
     }
 
     #[cfg(feature = "list_to_struct")]
@@ -238,7 +234,7 @@ impl ListNameSpace {
 
         Expr::Function {
             input: vec![self.0, other],
-            function: FunctionExpr::ListContains,
+            function: FunctionExpr::ListExpr(ListFunction::Contains),
             options: FunctionOptions {
                 collect_groups: ApplyOptions::ApplyFlat,
                 input_wildcard_expansion: true,
